#ifndef lint
static char *sccsid = "@(#)xcal.c	3.22 (Hillside Systems) 1/16/91";
static char *copyright = "@(#)Copyright 1989,1990 Peter Collinson, Hillside Systems";
#endif  /* lint */
/***

* program name:
	xcal.c
* function:
	display the current calendar date
	if pressed as a button go into strip calendar mode
* switches:
	-format str	use str as a display format
	-debug		run quickly incrementing time - 1 day per sec
	-alarmscan	print alarm debug info
* libraries used:
	libXaw.a, libXmu.a libXt.a libX11.a
* compile time parameters:
	standard
* history:
	Written November 1989
	Peter Collinson
	Hillside Systems
* (C) Copyright: 1989 Hillside Systems/Peter Collinson
	
	Permission to use, copy, modify, and distribute this software
	and its documentation for any purpose is hereby granted to
	anyone, provided that the above copyright notice appear
	in all copies and that both that copyright notice and this
	permission notice appear in supporting documentation, and that
	the name of Peter Collinson not be used in advertising or
	publicity pertaining to distribution of the software without
	specific, written prior permission.  Hillside Systems makes no
	representations about the suitability of this software for any
	purpose.  It is provided "as is" without express or implied
	warranty.
	
	Peter Collinson DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
	SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
	AND FITNESS, IN NO EVENT SHALL Peter Collinson BE LIABLE FOR
	ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
	WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
	WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
	ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
	PERFORMANCE OF THIS SOFTWARE.

***/
#include <stdio.h>
#include <ctype.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#if defined XI18N && ProjectX == 5
#include <X11/wchar.h>
#include <X11/Xaw/CommandLc.h>
#else
#include <X11/Xaw/Command.h>
#endif	
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Form.h>
/* + kataoka@crl.mei.co.jp */
#include <time.h>
/* + */
#include "xcal.h"

char	date_area[BUFSIZ];

/*	command line options specific to the application */
static XrmOptionDescRec Options[] = {
{"-debug",	"debug",	XrmoptionNoArg,	(caddr_t)"TRUE"},
{"-alarmscan",	"alarmScan",	XrmoptionNoArg,	(caddr_t)"TRUE"},
{"-format",	"format",	XrmoptionSepArg, NULL},
};

struct resources appResources;

Pixmap	MouseOnPix;
Pixmap	MouseOffPix;

#define offset(field) XtOffset(struct resources *, field)

static XtResource Resources[] = {
{"debug", "Debug", XtRBoolean, sizeof(Boolean),
	offset(debug), XtRString, "False" },
{"alarmScan", "AlarmScan", XtRBoolean, sizeof(Boolean),
	offset(alarmScan), XtRString, "False" },
{"reverseVideo", "ReverseVideo", XtRBoolean, sizeof(Boolean),
	offset(reverseVideo), XtRString, "False" },
{"xcalendarCompat", "XcalendarCompat", XtRBoolean, sizeof(Boolean),
	offset(calCompat), XtRString, "False" },
{"giveHelp", "GiveHelp", XtRBoolean, sizeof(Boolean),
	offset(giveHelp), XtRString, "True" },
{"useMemo", "UseMemo", XtRBoolean, sizeof(Boolean),
	offset(useMemo), XtRString, "True" },
{"memoLeft", "MemoLeft", XtRBoolean, sizeof(Boolean),
	offset(memoLeft), XtRString, "True" },
{"initialCalendar", "InitialCalendar", XtRBoolean, sizeof(Boolean),
	offset(initialCalendar), XtRString, "False" },
{"initialEdit", "InitialEdit", XtRBoolean, sizeof(Boolean),
	offset(initialEdit), XtRString, "False" },
{"initialMemo", "InitialMemo", XtRBoolean, sizeof(Boolean),
	offset(initialMemo), XtRString, "False" },
{"markForeground", "MarkForeground", XtRPixel, sizeof(Pixel),
	offset(marked.fg), XtRString, "White" },
{"markBackground", "MarkBackground", XtRPixel, sizeof(Pixel),
	offset(marked.bg), XtRString, "Black" },
{"markToday", "MarkToday", XtRBoolean, sizeof(Boolean),
	offset(markToday), XtRString, "True" },
#ifdef HOLIDAY
{"saturdayForeground", "SaturdayForeground", XtRPixel, sizeof(Pixel),
	offset(saturday.fg), XtRString, "Blue" },
{"saturdayBackground", "SaturdayBackground", XtRPixel, sizeof(Pixel),
	offset(saturday.bg), XtRString, "White" },
{"holidayForeground", "HolidayForeground", XtRPixel, sizeof(Pixel),
	offset(holiday.fg), XtRString, "Red" },
{"holidayBackground", "HolidayBackground", XtRPixel, sizeof(Pixel),
	offset(holiday.bg), XtRString, "White" },
{"holidays", "Holidays", XtRString, sizeof(String),
	offset(holidays), XtRString, ""},
# ifdef XI18N
{"fontSetSaturday", "FontSetSaturday", XtRFontSet, sizeof(XFontSet),
	offset(fontSetSaturday), XtRString, "-*--14-*"},
{"fontSetHoliday", "FontSetHoliday", XtRFontSet, sizeof(XFontSet),
	offset(fontSetHoliday), XtRString, "-*--14-*"},
# else
{"fontSaturday", "FontSaturday", XtRFontStruct, sizeof(XFontStruct *),
	offset(fontSaturday), XtRString, "XtDefaultFont"},
{"fontHoliday", "FontHoliday", XtRFontStruct, sizeof(XFontStruct *),
	offset(fontHoliday), XtRString, "XtDefaultFont"},
# endif
#else /* HOLIDAY */
# ifdef XI18N
{"fontSetToday", "FontSetToday", XtRFontSet, sizeof(XFontSet),
	offset(fontSetToday), XtRString, "-*--14-*"},
# else
{"fontToday", "FontToday", XtRFontStruct, sizeof(XFontStruct *),
	offset(fontToday), XtRString, "XtDefaultFont"},
# endif
{"todayForeground", "TodayForeground", XtRPixel, sizeof(Pixel),
	offset(today.fg), XtRString, "White" },
{"todayBackground", "TodayBackground", XtRPixel, sizeof(Pixel),
	offset(today.bg), XtRString, "Black" },
#endif /* HOLIDAY */
{"format", "Format",XtRString, sizeof(String),
	offset(opfmt), XtRString, "%W %M %D %Y"},
{"monthFormat", "MonthFormat", XtRString, sizeof(String),
	offset(monthOpfmt), XtRString, "%M %Y"},
{"dateYearIsTwoDigits", "DateYearIsTwoDigits", XtRBoolean, sizeof(Boolean),
	offset(dateYearIs2), XtRString, "False" },
{"monthYearIsTwoDigits", "MonthYearIsTwoDigits", XtRBoolean, sizeof(Boolean),
	offset(monthYearIs2), XtRString, "False" },
{"editYearIsTwoDigits", "EditYearIsTwoDigits", XtRBoolean, sizeof(Boolean),
	offset(editYearIs2), XtRString, "False" },
{"memoYearIsTwoDigits", "MemoYearIsTwoDigits", XtRBoolean, sizeof(Boolean),
	offset(memoYearIs2), XtRString, "False" },
#ifdef XI18N
{"i18nWmTitle", "I18nWmTitle", XtRBoolean, sizeof(Boolean),
	offset(i18nWmTitle), XtRString, "False" },
#endif
{"directory", "Directory", XtRString, sizeof(String),
	offset(directory), XtRString, "Calendar"},
{"textBufferSize", "TextBufferSize", XtRInt, sizeof(int),
	offset(textbufsz), XtRString, "2048"},
{"useWmTitle", "UseWmTitle", XtRBoolean, sizeof(Boolean),
	offset(useWmTitle), XtRString, "True"},
{"minStripWidth", "MinStripWidth", XtRDimension, sizeof(Dimension),
	offset(minstripwidth), XtRString, "0"},
{"january", "January", XtRString, sizeof(String),
	offset(mon[0]), XtRString, "January"},
{"february", "February", XtRString, sizeof(String),
	offset(mon[1]), XtRString, "February"},
{"march", "March", XtRString, sizeof(String),
	offset(mon[2]), XtRString, "March"},
{"april", "April", XtRString, sizeof(String),
	offset(mon[3]), XtRString, "April"},
{"may", "May", XtRString, sizeof(String),
	offset(mon[4]), XtRString, "May"},
{"june", "June", XtRString, sizeof(String),
	offset(mon[5]), XtRString, "June"},
{"july", "July", XtRString, sizeof(String),
	offset(mon[6]), XtRString, "July"},
{"august", "August", XtRString, sizeof(String),
	offset(mon[7]), XtRString, "August"},
{"september", "September", XtRString, sizeof(String),
	offset(mon[8]), XtRString, "September"},
{"october", "October", XtRString, sizeof(String),
	offset(mon[9]), XtRString, "October"},
{"november", "November", XtRString, sizeof(String),
	offset(mon[10]), XtRString, "November"},
{"december", "December", XtRString, sizeof(String),
	offset(mon[11]), XtRString, "December"},
{"jan", "Jan", XtRString, sizeof(String),
	offset(smon[0]), XtRString, "Jan"},
{"feb", "Feb", XtRString, sizeof(String),
	offset(smon[1]), XtRString, "Feb"},
{"mar", "Mar", XtRString, sizeof(String),
	offset(smon[2]), XtRString, "Mar"},
{"apr", "Apr", XtRString, sizeof(String),
	offset(smon[3]), XtRString, "Apr"},
{"may", "May", XtRString, sizeof(String),
	offset(smon[4]), XtRString, "May"},
{"jun", "Jun", XtRString, sizeof(String),
	offset(smon[5]), XtRString, "Jun"},
{"jul", "Jul", XtRString, sizeof(String),
	offset(smon[6]), XtRString, "Jul"},
{"aug", "Aug", XtRString, sizeof(String),
	offset(smon[7]), XtRString, "Aug"},
{"sep", "Sep", XtRString, sizeof(String),
	offset(smon[8]), XtRString, "Sep"},
{"oct", "Oct", XtRString, sizeof(String),
	offset(smon[9]), XtRString, "Oct"},
{"nov", "Nov", XtRString, sizeof(String),
	offset(smon[10]), XtRString, "Nov"},
{"dec", "Dec", XtRString, sizeof(String),
	offset(smon[11]), XtRString, "Dec"},
{"sunday", "Sunday", XtRString, sizeof(String),
	offset(day[0]), XtRString, "Sunday"},
{"monday", "Monday", XtRString, sizeof(String),
	offset(day[1]), XtRString, "Monday"},
{"tuesday", "Tuesday", XtRString, sizeof(String),
	offset(day[2]), XtRString, "Tuesday"},
{"wednesday", "Wednesday", XtRString, sizeof(String),
	offset(day[3]), XtRString, "Wednesday"},
{"thursday", "Thursday", XtRString, sizeof(String),
	offset(day[4]), XtRString, "Thursday"},
{"friday", "Friday", XtRString, sizeof(String),
	offset(day[5]), XtRString, "Friday"},
{"saturday", "Saturday", XtRString, sizeof(String),
	offset(day[6]), XtRString, "Saturday"},
{"sun", "Sun", XtRString, sizeof(String),
	offset(sday[0]), XtRString, "Sun"},
{"mon", "Mon", XtRString, sizeof(String),
	offset(sday[1]), XtRString, "Mon"},
{"tue", "Tue", XtRString, sizeof(String),
	offset(sday[2]), XtRString, "Tue"},
{"wed", "Wed", XtRString, sizeof(String),
	offset(sday[3]), XtRString, "Wed"},
{"thu", "Thu", XtRString, sizeof(String),
	offset(sday[4]), XtRString, "Thu"},
{"fri", "Fri", XtRString, sizeof(String),
	offset(sday[5]), XtRString, "Fri"},
{"sat", "Sat", XtRString, sizeof(String),
	offset(sday[6]), XtRString, "Sat"},
{"weekly", "Weekly", XtRString, sizeof(String),
	offset(weekly), XtRString, "Weekly"},
{"alarms", "Alarms", XtRBoolean, sizeof(Boolean),
	offset(alarms), XtRString, "True"},
{"update", "Update", XtRInt, sizeof(int),
	offset(update), XtRString, "0"},
{"volume", "Volume", XtRInt, sizeof(int),
	offset(volume), XtRString, "50"},
{"nbeeps", "Nbeeps", XtRInt, sizeof(int),
	offset(nbeeps), XtRString, "3"},
{"cmd", "Cmd", XtRString, sizeof(String),
	offset(cmd), XtRString, NULL},
{"countdown", "Countdown", XtRString, sizeof(String),
	offset(countdown), XtRString, "10,0"},
{"autoquit", "Autoquit",  XtRInt, sizeof(int),
	offset(autoquit), XtRString, "120"},
{"alarmleft", "Alarmleft",  XtRString, sizeof(String),
	offset(alarmleft), XtRString, "%d minutes before..."},
{"alarmnow", "Alarmnow",  XtRString, sizeof(String),
	offset(alarmnow), XtRString, "Time is now..."},
{"memoFile", "MemoFile", XtRString, sizeof(String),
	offset(memoFile), XtRString, "memo"},
{"maxDisplayLines", "MaxDisplayLines", XtRInt, sizeof(int),
	offset(maxDisplayLines), XtRString, "5"},
{"helpFile", "HelpFile", XtRString, sizeof(String),
	 offset(helpfile), XtRString, "/usr/lib/X11/xcal.hlp"},
};

static XtCallbackRec callbacks[] = {
   {NULL, NULL},
   {NULL, NULL},
};
#define ClearCallbacks() bzero((caddr_t)callbacks, sizeof (callbacks))

static XtActionsRec appActions[]= {
	{"setdate", SetDate},
	{"leave", AskLeave},
	{"SetDateAction", TextCal},
};

static String defTranslations =
	"<Btn2Down>: set()\n\
	<Btn2Up>:setdate() unset()\n\
	<Btn3Down>: set()\n\
	<Btn3Up>: leave() unset()";

static Arg	wargs[7] = {
	XtNlabel, (XtArgVal) date_area,
	XtNcallback, (XtArgVal)callbacks,
};

Widget	toplevel;

Date	today;

/*
 *	Forward routines local to this file
 */
static void	MkDate();
static void	DebugMkDate();
static void	DoTemplate();
static void	PixInit();

#include "mouse.bm"
#include "mouseaway.bm"

#ifdef XI18N
char *DayWeek[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
char *Months[]	= {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
		   "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
#endif

void
main(argc, argv)
	unsigned int argc;
	char **argv;
{
	Widget parent;
	Widget memo;
	Widget lab;
	
#ifdef XI18N
 	XtSetLanguageProc(NULL, NULL, NULL);
#endif

	toplevel = XtInitialize(argv[0], "XCal",
				Options, XtNumber(Options), &argc, argv);

	PixInit(toplevel);
	
	if (argc != 1)
		fprintf(stderr, "Error in arguments\n", argv[0]);

	XtGetApplicationResources(toplevel, (caddr_t)&appResources, Resources,
			      XtNumber(Resources), (ArgList)NULL, 0);

	/*
	 *	If reverse video
	 *	invert default colour settings
	 */
	if (appResources.reverseVideo)
	{	Colour	old;
		old = appResources.marked;
		appResources.marked.fg = old.bg;
		appResources.marked.bg = old.fg;
#ifdef HOLIDAY
		old = appResources.saturday;
		appResources.saturday.fg = old.bg;
		appResources.saturday.bg = old.fg;
		old = appResources.holiday;
		appResources.holiday.fg = old.bg;
		appResources.holiday.bg = old.fg;
#else
		old = appResources.today;
		appResources.today.fg = old.bg;
		appResources.today.bg = old.fg;
#endif
	}
	
	InitMonthEntries();
	
	DoTemplate();		/* give a maximum initial size of the box */

	/*
	 *	Top level widget is now a form
	 *	assuming that memo is wanted
	 */
	if (appResources.useMemo)
	{	parent = toplevel;
		XtSetArg(wargs[2], XtNborderWidth, 0);
		XtSetArg(wargs[3], XtNdefaultDistance, 0);
		parent = XtCreateManagedWidget("today", formWidgetClass,
					parent, &wargs[2], 3);

		if (appResources.memoLeft)
		{	XtSetArg(wargs[2], XtNfromHoriz, NULL);
			XtSetArg(wargs[3], XtNleft, XtChainLeft);
			XtSetArg(wargs[4], XtNright, XtRubber);
			XtSetArg(wargs[5], XtNborderWidth, 0);
			XtSetArg(wargs[6], XtNbitmap, MouseOnPix);
			callbacks[0].callback = DoMemo;
#ifdef XI18N
			memo = XtCreateManagedWidget("today", commandLocaleWidgetClass,
#else
			memo = XtCreateManagedWidget("today", commandWidgetClass,
#endif
					     parent, &wargs[1], 6);
			ClearCallbacks();

			XtSetArg(wargs[2], XtNfromHoriz, memo);
			XtSetArg(wargs[3], XtNleft, XtRubber);
			XtSetArg(wargs[4], XtNright, XtChainRight);
			XtSetArg(wargs[5], XtNborderWidth, 0);
			callbacks[0].callback = DoCalendar;
#ifdef XI18N
			lab = XtCreateManagedWidget("date", commandLocaleWidgetClass,
#else
			lab = XtCreateManagedWidget("date", commandWidgetClass,
#endif
					    parent, wargs, 6);
		}
		else
		{
			XtSetArg(wargs[2], XtNfromHoriz, NULL);
			XtSetArg(wargs[3], XtNleft, XtChainLeft);
			XtSetArg(wargs[4], XtNright, XtRubber);
			XtSetArg(wargs[5], XtNborderWidth, 0);
			callbacks[0].callback = DoCalendar;
#ifdef XI18N
			lab = XtCreateManagedWidget("date", commandLocaleWidgetClass,
#else
			lab = XtCreateManagedWidget("date", commandWidgetClass,
#endif
					    parent, wargs, 6);
			ClearCallbacks();

			XtSetArg(wargs[2], XtNfromHoriz, lab);
			XtSetArg(wargs[3], XtNleft, XtRubber);
			XtSetArg(wargs[4], XtNright, XtChainRight);
			XtSetArg(wargs[5], XtNborderWidth, 0);
			XtSetArg(wargs[6], XtNbitmap, MouseOnPix);
			callbacks[0].callback = DoMemo;
#ifdef XI18N
			memo = XtCreateManagedWidget("today", commandLocaleWidgetClass,
#else
			memo = XtCreateManagedWidget("today", commandWidgetClass,
#endif
					     parent, &wargs[1], 6);
		}
	}
	else
	{
		callbacks[0].callback = DoCalendar;
#ifdef XI18N
		lab = XtCreateManagedWidget("date", commandLocaleWidgetClass,
#else
		lab = XtCreateManagedWidget("date", commandWidgetClass,
#endif
					    toplevel, wargs, 2);
	}

	ClearCallbacks();

	XtSetMappedWhenManaged(toplevel, False);

	XtRealizeWidget(toplevel);	/* set the default geometry */
	
	if (appResources.debug)
	{	fprintf(stderr, "Debug ON\n");
		DebugMkDate(lab);
	}
	else	MkDate(lab);

/*
	if (appResources.giveHelp)
	{	printf("\
The small date strip is a button\n\
Enter the button showing the date and use\n\
the mouse buttons to select further actions:\n\
	Left mouse button pops up this month's calendar strip\n\
	Middle mouse button permits date selection\n\
	Right mouse button allows exit\n\
Click with the left mouse button in the small box holding\n\
the mouse icon to edit a memo file\n");
	}
*/
	XtAddActions(appActions, 3);	/* register actions */
	XtAugmentTranslations(lab, XtParseTranslationTable(defTranslations));
	if (appResources.useMemo)
		XtAugmentTranslations(memo, XtParseTranslationTable(defTranslations));

	XtMapWidget(toplevel);

	if (appResources.initialCalendar)
		DoCalendar(lab, NULL, NULL);


	if (appResources.initialEdit)
	{	MonthEntry	*me;

		me = GetMonthEntry(today.year, today.month);
		if (me->me_have[today.day])
			StartEditing(lab, &today);
	}

	if (appResources.useMemo && appResources.initialMemo)
		DoMemo(memo, NULL, NULL);
		
	InitAlarms();
	
	XtMainLoop();
	exit(0);
}

/*
 *	Initialise Pixmaps
 */
static void
PixInit(toplevel)
	Widget	toplevel;
{
	Display	*theDisplay = XtDisplay(toplevel);

	MouseOnPix = XCreateBitmapFromData(theDisplay,
			      DefaultRootWindow(theDisplay),
			      (char *)mouse_bits, mouse_width, mouse_height);
	MouseOffPix = XCreateBitmapFromData(theDisplay,
			      DefaultRootWindow(theDisplay),
			      (char *)mouseaway_bits, mouseaway_width, mouseaway_height);
}

/*
 *	Flip mouse state
 */
void
MouseShow(w, OnOff)
	Widget	w;
	Boolean OnOff;
{
	Arg	arg[1];

	XtSetArg(arg[0], XtNbitmap, OnOff ? MouseOnPix : MouseOffPix);
	XtSetValues(w, arg, 1);
}
 

/*
 *	Exit routine
 */
void
Leave(retval)
	int retval;
{	exit(retval);
}

/************************************************************************/
/*									*/
/*									*/
/*	This deals with the top level date `icon'			*/
/*									*/
/*									*/
/************************************************************************/

/*
 *	Time management code
 *	Set up a Date structure from today's information
 */
static void
ConvDate(tm, dp)
	struct	tm *tm;
	Date *dp;
{
	dp->day = tm->tm_mday;
	dp->month = tm->tm_mon;
	dp->year = tm->tm_year + 1900;
	dp->wday = tm->tm_wday;
}


static void
MkDate(w)
	Widget w;
{	long	ti;
	struct	tm *tm;
	static	timedOut;
	Date	yesterday;
	
	if (timedOut)
		yesterday = today;
		
	(void) time(&ti);
	tm = localtime(&ti);

	ConvDate(tm, &today);

	PlaceStr(date_area, &today, appResources.dateYearIs2, appResources.opfmt);

	XtSetValues(w, wargs, 1);

	if (timedOut)
	{	ChangeHighlight(&yesterday, &today);
		AlarmFilePoll(tm);
		UpdateMemo();
	}

	ti = 24*60*60 - (tm->tm_hour*60*60 + tm->tm_min*60 + tm->tm_sec);
	XtAddTimeOut(ti*1000, MkDate, (caddr_t)w);
	timedOut++;
}

static void
DebugMkDate(w)
	Widget w;
{	static long ti;
	struct	tm *tm;
	static	timedOut;
	Date	yesterday;
	
	if (timedOut)
		yesterday = today;
		
	if (ti == 0)
		(void) time(&ti);
	else	ti += 24*60*60;
	
	tm = localtime(&ti);
	ConvDate(tm, &today);
	
	PlaceStr(date_area, &today, appResources.dateYearIs2, appResources.opfmt);

	XtSetValues(w, wargs, 1);

	if (timedOut)
	{	ChangeHighlight(&yesterday, &today);
		AlarmFilePoll(tm);
		UpdateMemo();
	}

	XtAddTimeOut(2000, DebugMkDate, (caddr_t)w);
	timedOut++;
}

/*
 *	DoTemplate
 *	place an initial string into the date area so that the label
 *	box will always be big enough
 */
static void
DoTemplate()
{	int	max;
	int	i;
	int	len;
	char	trial[BUFSIZ];
	Date	da;

	da.day = 99;
	da.year = 9999;

	for (da.wday = max = i = 0; i < 7; i++)
	{	len = strlen(appResources.day[i]);
		if (len > max)
		{	max = len;
			da.wday = i;
		}
	}

	for (max = i = 0; i < 12; i++)
	{	da.month = i;
		PlaceStr(trial, &da, appResources.dateYearIs2, appResources.opfmt);
		len = strlen(trial);
		if (len > max)
		{	max = len;
			strcpy(date_area, trial);
		}
	}
}
	
/*
 *	make a string
 */
void
PlaceStr(dest, da, is2, fmt)
	String	dest;
	Date	*da;
	Boolean is2;
	String	fmt;
{
	int	y = da->year;
	char	*fp, *dp;

	if (y > 99 && y < 1900) y -= 100;
	if (is2) y %= 100;
		
	dp = dest;
	for(fp = fmt; *fp; fp++) {
		if (*fp == '%') {
			if (*++fp == '\0') break;

			switch (*fp) {
			case 'Y':
				sprintf(dp, "%d", y);
				dp += strlen(dp);
				break;
			case 'M':
				strcpy(dp, appResources.mon[da->month]);
				dp += strlen(dp);
				break;
			case 'D':
				sprintf(dp, "%d", da->day);
				dp += strlen(dp);
				break;
			case 'W':
				strcpy(dp, appResources.day[da->wday]);
				dp += strlen(dp);
				break;
			default:
				*(dp++) = *fp;
			}
		}
		else
			*(dp++) = *fp;
	}
	*dp = '\0';
}

