#ifndef lint
static char *sccsid = "@(#)xcal_strip.c	3.10 (Hillside Systems) 1/16/91";
static char *copyright = "@(#)Copyright 1989,1990 Peter Collinson, Hillside Systems";
#endif  /* lint */
/***

* module name:
	xcal_strip.c
* function:
	Deal with the popup strip calendars obtained either by
	selection and the middle button, or by the < and > buttons
	on each strip.
* history:
	Written November 1989
	Peter Collinson
	Hillside Systems
* (C) Copyright: 1989 Hillside Systems/Peter Collinson
	
	For full permissions and copyright notice - see xcal.c
***/
#include <stdio.h>
#include <ctype.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#if defined XI18N && ProjectX == 5
#include <X11/wchar.h>
#endif
#include <X11/Xaw/Paned.h>
#if defined XI18N && ProjectX == 5
#include <X11/Xaw/CommandLc.h>
#else
#include <X11/Xaw/Command.h>
#endif
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/AsciiText.h>
#include "xcal.h"

static XtCallbackRec callbacks[] = {
   {NULL,NULL},
   {NULL,NULL},
   {NULL,NULL},
   {NULL,NULL}
};
#define ClearCallbacks() bzero((caddr_t)callbacks, sizeof (callbacks))

Date	callb;		/* contains date when calendar day button pressed */
#ifdef FAKE_COMMANDLC_LABEL_HEIGHT
Dimension	monInfoHoffset, weekInfoHoffset;
Dimension	monInfoHdelta = 0, weekInfoHdelta = 0;
#endif

/*
 *	Forward routines local to this file
 */
void MakeMonth();
void	DayBack();
#ifndef LONG_IS_32_BITS
void	YmBack();
#endif
void	StripQuit();
void	StripHelp();
void	WeeklyHelp();
void	MakeNewMonth();

Cardinal DateSum();
Cardinal NumberOfDays();
Cardinal FirstDay();
Cardinal JanuaryOne();

/*
 *	Local routines
 */
static void	CreateActionBar();
static void	CreateWeeklyActionBar();

/*
 *	Start a strip calendar happening
 *	a callback of left button 
 */
/* ARGSUSED */
void
DoCalendar(w, closure, call_data)
	Widget w;
	caddr_t	closure;
	caddr_t call_data;
{
	NewMonthStrip(&today);		/* today is global */
}

/* ARGSUSED */
void
DoWeekly(w, closure, call_data)
	Widget w;
	caddr_t	closure;
	caddr_t call_data;
{
	Date	thisday;

	thisday.day = 0;
	thisday.month = 0;
	thisday.year = 0;
	thisday.wday = 0;
	NewMonthStrip(&thisday);	/* today is global */
}


/*
 *	Start a strip calendar happening
 *	a callback of the > or < buttons in another strip
 */
/* ARGSUSED */
static void
MakeNewMonth(w, closure, call_data)
	Widget w;
	caddr_t	closure;
	caddr_t call_data;
{
	Date	thisday;

	thisday.year = YrUnpack((Cardinal)closure);
	thisday.month = MoUnpack((Cardinal)closure);
	thisday.day = today.day;
	NewMonthStrip(&thisday);
}

#ifdef HOLIDAY
static int
_isholiday(d, m)
	int d, m;
{
	char *p, pattern[5];
	char *hp;
	
	sprintf(pattern, "%02d%02d", m+1, d);
	p = pattern;
	for (hp = appResources.holidays; *hp; hp++) {
		if (*p != *hp)
			p = pattern;
		else 
			if (!*++p) return True;
	}
	return False;
}

static int
isholiday(dw, d, m, y)
	int dw, d, m, y;
{
	if (_isholiday(d, m))
		return True;

	/* a holiday in lieu */
	for(;;) {
		/* go back to yesterday */
		dw = (dw-1+7)%7;
		d--;
		if (d == 0) {
			m = (m-1+12)%12;
			if (m == 11) /* 11 is December !! */
				y--;
			d = NumberOfDays(m, y);
		}

		if (!_isholiday(d, m))
			return False;
		else if (dw == 0) /* Sunday */
			return True;
	}
}

ReverseVideoWidget(w)
	Widget w;
{
	Pixel	fg, bg;
	Arg	args[2];
	
	XtSetArg(args[0], XtNforeground, &fg);
	XtSetArg(args[1], XtNbackground, &bg);
	XtGetValues(w, args, 2);
	XtSetArg(args[0], XtNforeground, bg);
	XtSetArg(args[1], XtNbackground, fg);
	XtSetValues(w, args, 2);
}
#endif /* HOLIDAY */

/* 
 *	Do all the X stuff to popup a Strip calendar
 *	A calendar strip is:
 *
 *	Popup ("<month year>")		// Name is the month and the year
 *	    Paned ("<month>")		// Name is the month
 *		Label ("header")	// optional contains Month Year
 *		Form ("action")		// < Quit >
 *		    Command ("back")	// contains < 
 *		    Label ("quit")	// contains Quit
 *		    Command ("next")	// contains >
 *	(Then many of..)
 *		Form ("<dd DDD>")	// where dd is the day number, DDD is the
 *					// day of the week
 *		    Label ("label")	// contains the string above
 *		    Command ("info")	// contains the text from the file
 *		
 */
void
NewMonthStrip(td)
	Date	*td;
{
	Widget		shell, mon, dw, lw, lwi, form;
	Arg		args[15];
	char		nbuf[256];
	char		iconName[80];
	int		type;
	MonthEntry	*me;
	Instance	*ins;
	register int	i;
	register Cardinal nargs;
	Cardinal	thisDay;
	Cardinal	startLoop;
	String		dayStr;
	Cardinal	numberOfDays;
	Boolean		defaultsAreSet = False;
	Boolean		markThisMonth = False;
	Cardinal	adjustLabelY;
	Cardinal	adjustInfoY;
	Dimension	labelH, infoH;
	Dimension	width;
	Dimension	totalWidth;
	char		*formName = "   ";

	type = (td->day == 0) ? ME_WEEKLY : ME_MONTHLY;
	
	/*
	 *	There are lots of differences between
	 *	Months and weekly strips here.
	 *	Later tests are done using a switch structure
	 */
	switch (type)
	{
	case ME_MONTHLY:
		/*
		 *	Title bar is month and date
		 */
		PlaceStr(nbuf, td, appResources.monthYearIs2, appResources.monthOpfmt);
#ifdef XI18N
		if (appResources.i18nWmTitle)
			(void) strcpy(iconName, nbuf);
		else
			sprintf(iconName, "%s %d", Months[td->month],
				appResources.monthYearIs2 ?
				td->year%100 : td->year);
#else
		(void) sprintf(iconName, "%s %d", appResources.smon[td->month], td->year);
#endif
		XtSetArg(args[0], XtNiconName, iconName);
		shell = XtCreatePopupShell("month", topLevelShellWidgetClass, toplevel, args, 1);
		ins = RegisterMonth(td->year, td->month, shell);	
		XtSetArg(args[0], XtNresizable, True);
		mon = XtCreateManagedWidget("panel", panedWidgetClass, shell, args, 1);
		thisDay = FirstDay(td->month, td->year);
		numberOfDays = NumberOfDays(td->month, td->year);
		startLoop = 1;
		/*
		 *	Get the map for this year
		 */
		me = GetMonthEntry(td->year, td->month);
		me->me_type = type;
		/*
		 *	see if we will need to worry about marking today's entry
		 */
		if (appResources.markToday && td->year == today.year && td->month == today.month)
			markThisMonth = True;
		break;
	case ME_WEEKLY:
#ifdef XI18N
		if (appResources.i18nWmTitle)
			(void) strcpy(iconName, appResources.weekly);
		else
			strcpy(iconName, "Weekly");
#else
		(void) strcpy(iconName, appResources.weekly);
#endif
		XtSetArg(args[0], XtNiconName, iconName);
		shell = XtCreatePopupShell("weekly", topLevelShellWidgetClass, toplevel, args, 1);
		ins = RegisterMonth(0, 0, shell);	
		XtSetArg(args[0], XtNresizable, True);
		mon = XtCreateManagedWidget("panel", panedWidgetClass, shell, args, 1);
		thisDay = 0;
		numberOfDays = 6;	/* test is <= */
		startLoop = 0;
		/*
		 *	Get the map for this year
		 */
		me = GetWeeklyEntry();
		me->me_type = type;
		/*
		 *	Title bar is from the resources
		 */
		strcpy(nbuf, appResources.weekly);
		/*
		 *	see if we will need to worry about marking today's entry
		 */
		if (appResources.markToday)
			markThisMonth = True;
		break;
	}
	/*
	 *	Find size of title bar
	 *	by creating the widget and then throwing it away
	 */
	XtSetArg(args[0], XtNlabel, "mmmmmmmmm NNNN");
	lw = XtCreateManagedWidget("sizer", labelWidgetClass, shell, args, 1);
	XtSetArg(args[0], XtNwidth, &totalWidth);
	XtGetValues(lw, args, 1);
	XtUnmanageChildren(&lw, 1);
	XtDestroyWidget(lw);
	/*
	 *	Width is affected by a resource value
	 */
	if (appResources.minstripwidth && appResources.minstripwidth > totalWidth)
		totalWidth = appResources.minstripwidth;
	/*
	 *	Now set the title bar should we need it
	 */
	if (appResources.useWmTitle)
	{	XtSetArg(args[0], XtNlabel, XtNewString(nbuf));
		(void) XtCreateManagedWidget("header", labelWidgetClass, mon, args, 1);
	}
		
	/*
	 *	Action bar
	 */
	nargs = 0;
	XtSetArg(args[nargs], XtNshowGrip, False); nargs++;
	XtSetArg(args[nargs], XtNdefaultDistance, 2); nargs++;
	dw = XtCreateManagedWidget("action", formWidgetClass, mon, args, nargs);

	switch (type)
	{
	case ME_MONTHLY:
		CreateActionBar(shell, dw, mon, td);
		break;
	case ME_WEEKLY:
		CreateWeeklyActionBar(shell, dw);
		break;
	}

#ifdef	LONG_IS_32_BITS
	callbacks[0].callback = DayBack;
#else
	callbacks[0].callback = YmBack;
	callbacks[1].callback = DayBack;
#endif
	for (i = startLoop; i <= numberOfDays; i++)
	{
		dayStr = appResources.sday[thisDay];
		switch(type)
		{
		case ME_MONTHLY:
			(void) sprintf(nbuf, "%2d %s", i, dayStr);
			break;
		case ME_WEEKLY:
			(void) strcpy(nbuf, dayStr);
			break;
		}
#ifdef LONG_IS_32_BITS
		callbacks[0].closure = (caddr_t)DatePack(thisDay, i, td->month, td->year);
#else
		callbacks[0].closure = (caddr_t)DatePack(td->month, td->year);
		callbacks[1].closure = (caddr_t)DayPack(thisDay, i);
#endif		

#ifndef HOLIDAY
		thisDay = (thisDay+1)%7;
#endif
		
		/*
		 *	Each line in the strip is
		 *	form containing
		 *	label - command
		 */
		nargs = 0;
		XtSetArg(args[nargs], XtNshowGrip, False); nargs++;
		XtSetArg(args[nargs], XtNdefaultDistance, 0); nargs++;
		XtSetArg(args[nargs], XtNallowResize, True); nargs++;
		switch (type) {
		case ME_MONTHLY:
			sprintf(formName, "%d", i);
			break;
		case ME_WEEKLY:
#ifdef XI18N
			formName = DayWeek[i];
#else
			formName = appResources.smon[i];
#endif
			break;
		}

		ins->i_day[i] = form = XtCreateManagedWidget(formName, formWidgetClass, mon, args, nargs);
		nargs = 0;
		XtSetArg(args[nargs], XtNlabel, XtNewString(nbuf)); nargs++;
						/* a little naughty here */
						/* this string memory is lost */
						/* on quit */
		XtSetArg(args[nargs], XtNborderWidth, 0); nargs++;
		XtSetArg(args[nargs], XtNjustify, XtJustifyLeft); nargs++;
		XtSetArg(args[nargs], XtNfromHoriz, NULL); nargs++;
		XtSetArg(args[nargs], XtNleft, XtChainLeft); nargs++;
		XtSetArg(args[nargs], XtNright, XtChainLeft); nargs++;
		XtSetArg(args[nargs], XtNbottom, XtChainBottom); nargs++;
		
		ins->i_day_label[i] = lw = XtCreateManagedWidget("label", labelWidgetClass, form, args, nargs);

#ifdef HOLIDAY
		if (thisDay == 0 /* Sunday */ ||
		    (type == ME_MONTHLY
		     && isholiday(thisDay, i, td->month, td->year))){
			nargs = 0;
			XtSetArg(args[nargs], XtNforeground, appResources.holiday.fg); nargs++;
			XtSetArg(args[nargs], XtNbackground, appResources.holiday.bg); nargs++;
# ifdef XI18N
			XtSetArg(args[nargs], XtNfontSet, appResources.fontSetHoliday); nargs++;
# else
			XtSetArg(args[nargs], XtNfont, appResources.fontHoliday); nargs++;
# endif
			XtSetValues(lw, args, nargs);
		}
		else if (thisDay == 6){ /* Saturday */
			nargs = 0;
			XtSetArg(args[nargs], XtNforeground, appResources.saturday.fg); nargs++;
			XtSetArg(args[nargs], XtNbackground, appResources.saturday.bg); nargs++;
# ifdef XI18N
			XtSetArg(args[nargs], XtNfontSet, appResources.fontSetSaturday); nargs++;
# else
			XtSetArg(args[nargs], XtNfont, appResources.fontSaturday); nargs++;
# endif
			XtSetValues(lw, args, nargs);
		}
		thisDay = (thisDay+1)%7;
#endif /* HOLIDAY */

		/*
		 *	To get a handle on the old values which are lost by
		 *	highlighting we get them after we have created the
		 *	widget. Then we highlight today.
		 */
		if (markThisMonth &&
		    ((type == ME_MONTHLY && today.day == i) ||
		     (type == ME_WEEKLY && today.wday == i)))
		{
#ifdef HOLIDAY
			ReverseVideoWidget(lw);
#else /* HOLIDAY */
			nargs = 0;
			XtSetArg(args[nargs], XtNforeground, &ins->i_col.fg); nargs++;
			XtSetArg(args[nargs], XtNbackground, &ins->i_col.bg); nargs++;
# ifdef XI18N
			XtSetArg(args[nargs], XtNfontSet, &ins->i_font); nargs++;
# else
			XtSetArg(args[nargs], XtNfont, &ins->i_font); nargs++;
# endif
			XtGetValues(lw, args, nargs);

			nargs = 0;
			XtSetArg(args[nargs], XtNforeground, appResources.today.fg); nargs++;
			XtSetArg(args[nargs], XtNbackground, appResources.today.bg); nargs++;
# ifdef XI18N
			XtSetArg(args[nargs], XtNfontSet, appResources.fontSetToday); nargs++;
# else
			XtSetArg(args[nargs], XtNfont, appResources.fontToday); nargs++;
# endif
			XtSetValues(lw, args, nargs);
#endif /* HOLIDAY */
		}
		/*
		 *	Done the first time through
		 *	Gets the width of the line we have just made
		 */
		if (defaultsAreSet == False)
		{	/* compute text width */
			nargs = 0;
			XtSetArg(args[nargs], XtNwidth, &width); nargs++;
			XtSetArg(args[nargs], XtNheight, &labelH); nargs++;
			XtGetValues(lw, args, nargs);
			defaultsAreSet = True;
		}

#ifdef FAKE_COMMANDLC_LABEL_HEIGHT
		if (type == ME_MONTHLY && !monInfoHdelta) 
			GetInfoHeight(&monInfoHoffset, &monInfoHdelta, form);
		else if (type == ME_WEEKLY && !weekInfoHdelta) 
			GetInfoHeight(&weekInfoHoffset, &weekInfoHdelta, form);
#endif /* FAKE_COMMANDLC_LABEL_HEIGHT */

		/*
		 *	Start processing the RHS of the line
		 *	This contains text from the file
		 *	should any exist
		 */
		nargs = 0;
		XtSetArg(args[nargs], XtNborderWidth, 0); nargs++;
		XtSetArg(args[nargs], XtNcallback, callbacks); nargs++;
		XtSetArg(args[nargs], XtNfromHoriz, lw); nargs++;
		XtSetArg(args[nargs], XtNleft, XtChainLeft); nargs++;
		XtSetArg(args[nargs], XtNright, XtChainRight); nargs++;
		XtSetArg(args[nargs], XtNjustify, XtJustifyLeft); nargs++;
		XtSetArg(args[nargs], XtNwidth, totalWidth - width); nargs++;
		XtSetArg(args[nargs], XtNbottom, XtChainBottom); nargs++;
		XtSetArg(args[nargs], XtNresizable, True); nargs++;

		if (me->me_have[i])
		{
#ifndef HOLIDAY
			XtSetArg(args[nargs], XtNforeground, appResources.marked.fg); nargs++;
			XtSetArg(args[nargs], XtNbackground, appResources.marked.bg); nargs++;
#endif
#if defined(XI18N) && defined(FAKE_COMMANDLC_LABEL)
			XtSetArg(args[nargs], XtNlabel, fake_commandlc_label(me->me_have[i])); nargs++;
#else
			XtSetArg(args[nargs], XtNlabel, me->me_have[i]); nargs++;
#endif /* XI18N && FAKE_COMMANDLC_LABEL */
		}
		else
		{	XtSetArg(args[nargs], XtNlabel, "    "); nargs++;	}
#ifdef XI18N
		ins->i_day_info[i] = lwi = XtCreateManagedWidget("info", commandLocaleWidgetClass, form, args, nargs);
#else
		ins->i_day_info[i] = lwi = XtCreateManagedWidget("info", commandWidgetClass, form, args, nargs);
#endif

#ifdef HOLIDAY
		if (me->me_have[i]) {
			XtSetArg(args[0], XtNforeground, &ins->i_col.fg);
			XtSetArg(args[1], XtNbackground, &ins->i_col.bg);
			XtGetValues(lwi, args, 2);
			XtSetArg(args[0], XtNforeground, appResources.marked.fg);
			XtSetArg(args[1], XtNbackground, appResources.marked.bg);
			XtSetValues(lwi, args, 2);
		}
#endif

		/* deal with height */
		XtSetArg(args[0], XtNheight, &infoH);
		XtGetValues(lwi, args, 1);
#ifdef NEVER
		if (labelH < infoH)
		{	adjustLabelY = ((infoH-labelH)/2);
			/* fix up widget */
			nargs = 0;
			XtSetArg(args[nargs], XtNvertDistance, adjustLabelY); nargs++;
                        XtSetArg(args[nargs], XtNfromVert, NULL); nargs++;
			XtSetValues(lw, args, nargs);
		}
		else if (labelH > infoH)
		{	adjustInfoY = ((labelH - infoH)/2);
			/* fix up widget 1 */
			nargs = 0;
			XtSetArg(args[nargs], XtNvertDistance, adjustInfoY); nargs++;
                        XtSetArg(args[nargs], XtNfromVert, NULL); nargs++;
			XtSetValues(lwi, args, nargs);
		}
#endif

		/*
		 *	cope with 1752
		 */
		if (td->year == 1752 && td->month == 8 && i == 2)
		{	i = 13;
			numberOfDays += 11;	/* giving back the 11 days */
		}
	}
	ClearCallbacks();

	XtPopup(shell, XtGrabNone);
}

#ifdef FAKE_COMMANDLC_LABEL_HEIGHT
GetInfoHeight(offsetp, deltap, form)
	Dimension	*offsetp, *deltap;
	Widget		form;
{
	Widget	lwi;
	Arg	args[1];
	Dimension	infoH;

	XtSetArg(args[0], XtNlabel, "x");
# ifdef XI18N
	lwi = XtCreateManagedWidget("info", commandLocaleWidgetClass, form, args, 1);
# else
	lwi = XtCreateManagedWidget("info", commandWidgetClass, form, form, 1);
# endif
	XtSetArg(args[0], XtNheight, offsetp);
	XtGetValues(lwi, args, 1);
	XtUnmanageChildren(&lwi, 1);
	XtDestroyWidget(lwi);

	XtSetArg(args[0], XtNlabel, "x\nx");
# ifdef XI18N
	lwi = XtCreateManagedWidget("info", commandLocaleWidgetClass, form, args, 1);
# else
	lwi = XtCreateManagedWidget("info", commandWidgetClass, form, form, 1);
# endif
	XtSetArg(args[0], XtNheight, &infoH);
	XtGetValues(lwi, args, 1);
	XtUnmanageChildren(&lwi, 1);
	XtDestroyWidget(lwi);
	*deltap = infoH - *offsetp;
}
#endif /* FAKE_COMMANDLC_LABEL_HEIGHT */

/*
 *	Create action bar for normal monthly strip
 */
static void
CreateActionBar(shell, dw, mon, td)
	Widget	shell;
	Widget	dw;
	Widget	mon;
	Date	*td;
{
	Widget		lw;
	register Cardinal nargs;
	Arg		args[8];

	/*
	 *	back one month
	 *	label "<" from resources
	 */
	callbacks[0].callback = MakeNewMonth;
	callbacks[0].closure = (caddr_t)DateSum(td, -1);
	nargs = 0;
	XtSetArg(args[nargs], XtNcallback, callbacks); nargs++;
	XtSetArg(args[nargs], XtNfromHoriz, NULL); nargs++;
	XtSetArg(args[nargs], XtNleft, XtChainLeft); nargs++;
	XtSetArg(args[nargs], XtNright, XtChainLeft); nargs++;
#ifdef XI18N
	lw = XtCreateManagedWidget("back", commandLocaleWidgetClass, dw, args, nargs);
#else
	lw = XtCreateManagedWidget("back", commandWidgetClass, dw, args, nargs);
#endif
	ClearCallbacks();

	/*
	 *	Quit button
	 *	label "quit" from resources
	 */
	callbacks[0].callback = StripQuit;
	callbacks[0].closure = (caddr_t)shell;
	nargs = 0;
	XtSetArg(args[nargs], XtNcallback, callbacks); nargs++;
	XtSetArg(args[nargs], XtNfromHoriz, lw); nargs++;
	XtSetArg(args[nargs], XtNleft, XtChainLeft); nargs++;
	XtSetArg(args[nargs], XtNright, XtChainRight); nargs++;
#ifdef XI18N
	lw = XtCreateManagedWidget("quit", commandLocaleWidgetClass, dw, args, nargs);
#else
	lw = XtCreateManagedWidget("quit", commandWidgetClass, dw, args, nargs);
#endif
	ClearCallbacks();

	/*
	 *	Help button
	 *	label help from resources
	 */
	if (appResources.giveHelp) {	
		callbacks[0].callback = StripHelp;
		callbacks[0].closure = (caddr_t)0;
		nargs = 0;
		XtSetArg(args[nargs], XtNcallback, callbacks); nargs++;
		XtSetArg(args[nargs], XtNfromHoriz, lw); nargs++;
		XtSetArg(args[nargs], XtNleft, XtChainRight); nargs++;
		XtSetArg(args[nargs], XtNright, XtChainRight); nargs++;
#ifdef XI18N
		lw = XtCreateManagedWidget("help", commandLocaleWidgetClass, dw, args, nargs);
#else
		lw = XtCreateManagedWidget("help", commandWidgetClass, dw, args, nargs);
#endif
		ClearCallbacks();
	}

	/*
	 *	On one month
	 *	label ">" from reources
	 */
	callbacks[0].callback = MakeNewMonth;
	callbacks[0].closure = (caddr_t)DateSum(td, 1);
	nargs = 0;
	XtSetArg(args[nargs], XtNcallback, callbacks); nargs++;
	XtSetArg(args[nargs], XtNfromHoriz, lw); nargs++;
	XtSetArg(args[nargs], XtNleft, XtChainRight); nargs++;
	XtSetArg(args[nargs], XtNright, XtChainRight); nargs++;
#ifdef XI18N
	lw = XtCreateManagedWidget("next", commandLocaleWidgetClass, dw, args, nargs);
#else
	lw = XtCreateManagedWidget("next", commandWidgetClass, dw, args, nargs);
#endif
	ClearCallbacks();
}

/*
 *	Create action bar for normal monthly strip
 */
static void
CreateWeeklyActionBar(shell, dw)
	Widget	shell;
	Widget	dw;
{
	Widget		lw;
	register Cardinal nargs;
	Arg		args[8];

	/*
	 *	Quit button
	 *	label "quit" from resources
	 */
	callbacks[0].callback = StripQuit;
	callbacks[0].closure = (caddr_t)shell;
	nargs = 0;
	XtSetArg(args[nargs], XtNcallback, callbacks); nargs++;
	XtSetArg(args[nargs], XtNfromHoriz, NULL); nargs++;
	XtSetArg(args[nargs], XtNleft, XtChainLeft); nargs++;
	XtSetArg(args[nargs], XtNright, XtChainLeft); nargs++;
#ifdef XI18N
	lw = XtCreateManagedWidget("quit", commandLocaleWidgetClass, dw, args, nargs);
#else
	lw = XtCreateManagedWidget("quit", commandWidgetClass, dw, args, nargs);
#endif
	ClearCallbacks();

	/*
	 *	Help button
	 *	label help from resources
	 */
	if (appResources.giveHelp)
	{	
		callbacks[0].callback = WeeklyHelp;
		callbacks[0].closure = (caddr_t)0;
		nargs = 0;
		XtSetArg(args[nargs], XtNcallback, callbacks); nargs++;
		XtSetArg(args[nargs], XtNfromHoriz, lw); nargs++;
		XtSetArg(args[nargs], XtNleft, XtChainLeft); nargs++;
		XtSetArg(args[nargs], XtNright, XtChainLeft); nargs++;
#ifdef XI18N
		lw = XtCreateManagedWidget("help", commandLocaleWidgetClass, dw, args, nargs);
#else
		lw = XtCreateManagedWidget("help", commandWidgetClass, dw, args, nargs);
#endif
		ClearCallbacks();
	}
}

/*
 *	Called when the date changes to ensure that
 *	the correct day has the appropriate highlights
 */
void
ChangeHighlight(old, new)
	Date	*old;
	Date	*new;
{
	register Instance *ins;
	Arg		args[5];
	Cardinal	nargs;

	for (ins = FindInstanceList(old); ins; ins = ins->i_next)
	{
#ifdef HOLIDAY
		ReverseVideoWidget(ins->i_day_label[old->day]);
#else /* HOLIDAY */
		nargs = 0;	
		XtSetArg(args[nargs], XtNforeground, ins->i_col.fg); nargs++;
		XtSetArg(args[nargs], XtNbackground, ins->i_col.bg); nargs++;
# ifdef XI18N
		XtSetArg(args[nargs], XtNfontSet, ins->i_font); nargs++;
# else
		XtSetArg(args[nargs], XtNfont, ins->i_font); nargs++;
# endif
		XtSetValues(ins->i_day_label[old->day], args, nargs);
#endif /* HOLIDAY */
	}

	for (ins = FindInstanceList(new); ins; ins = ins->i_next)
	{
#ifdef HOLIDAY
		ReverseVideoWidget(ins->i_day_label[new->day]);
#else /* HOLIDAY */
		nargs = 0;
		XtSetArg(args[nargs], XtNforeground, &ins->i_col.fg); nargs++;
		XtSetArg(args[nargs], XtNbackground, &ins->i_col.bg); nargs++;
# ifdef XI18N
		XtSetArg(args[nargs], XtNfontSet, &ins->i_font); nargs++;
# else
		XtSetArg(args[nargs], XtNfont, &ins->i_font); nargs++;
# endif
		XtGetValues(ins->i_day_label[new->day], args, nargs);

		nargs = 0;	
		XtSetArg(args[nargs], XtNforeground, appResources.today.fg); nargs++;
		XtSetArg(args[nargs], XtNbackground, appResources.today.bg); nargs++;
# ifdef XI18N
		XtSetArg(args[nargs], XtNfontSet, appResources.fontSetToday); nargs++;
# else
		XtSetArg(args[nargs], XtNfont, appResources.fontToday); nargs++;
# endif
		XtSetValues(ins->i_day_label[new->day], args, nargs);
#endif /* HOLIDAY */
	}
}

/*
 *	Call back from a quit button to lose a month strip
 */
/* ARGSUSED */
static void
StripQuit(w, closure, call_data)
	Widget w;
	caddr_t	closure;
	caddr_t call_data;
{
	XtPopdown((Widget)closure);
	XtDestroyWidget((Widget)closure);
}
	

/*
 *	Month arithmetic and packing
 */
static Cardinal
DateSum(td, inx)
	Date	*td;
	int	inx;
{
	int	m, y;

	m = td->month;
	y = td->year;
	m += inx;
	if (m < 0)
	{	m = 11;
		y--;
	}
	else
	if (m > 11)
	{	m = 0;
		y++;
	}
#ifdef LONG_IS_32_BITS
	return(DatePack(0, 0, m, y));
#else
	return(DatePack(m, y));
#endif
}

/*
 *	Call back from day selection button press
 *	This is done in two stages if cannot fold dates into a closure
 */
/* ARGSUSED */
static void
DayBack(w, closure, call_data)
	Widget w;
	caddr_t	closure;
	caddr_t call_data;
{
#ifdef LONG_IS_32_BITS
	callb.month = MoUnpack((Cardinal)closure);
	callb.year = YrUnpack((Cardinal)closure);
#endif
	callb.day = DyUnpack((Cardinal)closure);
	callb.wday = WdUnpack((Cardinal)closure);
	StartEditing(w, &callb);
}

#ifndef LONG_IS_32_BITS
/* ARGSUSED */
static void
YmBack(w, closure, call_data)
	Widget w;
	caddr_t	closure;
	caddr_t call_data;
{
	callb.month = MoUnpack((Cardinal)closure);
	callb.year = YrUnpack((Cardinal)closure);
}
#endif

/*
 *	Stolen from xcalendar.c
 */
/* taken from cal.c */

char	mon[] = {
   31, 29, 31, 30,
   31, 30, 31, 31,
   30, 31, 30, 31,
};

static Cardinal calInit = 0;

static Cardinal
NumberOfDays(m, y)
	Cardinal m, y;
{
	if(calInit != y)
		(void) FirstDay(m, y); /* set side effect */
	return mon[m];
}

/* should be called first */
static Cardinal
FirstDay(m, y)
     Cardinal m, y;
{
	register d, i;

	calInit = y;
	d = JanuaryOne(y);
	mon[1] = 29;
	mon[8] = 30;
   
	switch((JanuaryOne(y+1)+7-d)%7)
	{
      
	/*
	 *	non-leap year
	 */
	case 1:
		mon[1] = 28;
		break;
      
	/*
	 *	1752
	 */
	default:
		mon[8] = 19;
		break;

	/*
	 *	leap year
	 */
	case 2:
		;
	}
   
	for(i=0; i<m; i++)
		d += mon[i];

	return(d%7);
}

/*
 *	return day of the week
 *	of jan 1 of given year
 */
static Cardinal
JanuaryOne(yr)
	Cardinal yr;
{
	register Cardinal y, d;

/*
 *	normal gregorian calendar
 *	one extra day per four years
 */

	y = yr;
	d = 4+y+(y+3)/4;

/*
 *	julian calendar
 *	regular gregorian
 *	less three days per 400
 */

	if(y > 1800) {
		d -= (y-1701)/100;
		d += (y-1601)/400;
	}

/*
 *	great calendar changeover instant
 */

	if(y > 1752)
		d += 3;

	return(d%7);
}
